#include "masking.h"

uint32_t uint32_rand() {
  return lrand48();
}

void bint32_and(bint32_t rc, bint32_t a,bint32_t b) {
  int i, j;
  uint32_t zij, zji, tmp;
  bint32_t r;

  // Diagonal
  for (i = 0; i < NUM_SHARES; i++) {
    r[i] = a[i] & b[i];
  }

  // Triangles + Row-Wise Sums
  for (i = 0; i < NUM_SHARES; i++) {
    for (j = i + 1; j < NUM_SHARES; j++) {
      zij  = uint32_rand();
      zji  = a[i] & b[j];
      zji  ^= zij;
      tmp  = a[j] & b[i];
      zji  ^= tmp;
      r[i] ^= zij;
      r[j] ^= zji;
    }
  }

  bint32_copy(rc,r);

  return;
}

void bint32_refresh(bint32_t r, bint32_t a) {
  int i, j;
  uint32_t aux;

  bint32_copy(r,a);

  for (i = 0; i < NUM_SHARES; i++) {
    for (j = i + 1; j < NUM_SHARES; j++) {
      aux = uint32_rand();
      r[i] ^= aux;
      r[j] ^= aux;
    }
  }

  return;
}

void bint32_full_refresh(bint32_t r, bint32_t a) {
  int i, j;
  uint32_t aux;

  bint32_copy(r,a);

  for (i = 0; i < NUM_SHARES; i++) {
    for (j = 0; j < NUM_SHARES; j++) {
      aux = uint32_rand();
      r[0] ^= aux;
      r[j] ^= aux;
    }
  }

  return;
}

void bint32_refresh_modp(bint32_t r, bint32_t a) {
  int i, j;
  uint32_t aux;

  bint32_copy(r,a);

  for (i = 0; i < NUM_SHARES; i++) {
    for (j = i + 1; j < NUM_SHARES; j++) {
      aux = uint32_rand_modp();
      r[i] = (r[i] + aux) % Q;
      r[j] = (r[j] + Q - aux) % Q;
    }
  }
  return;
}

// Compute the addition of x and y, store the result in res.
// The addition is computed module 2^32

void bint32_add (bint32_t res, bint32_t x, bint32_t y) {
  bint32_t aux,aux0;
  bint32_t p,g;
  int j, pow;

  bint32_xor(p, x, y);
  bint32_and(g, x, y);
  for(j = 1; j < LOG_BITSIZE; j++) {
    pow = 1 << (j - 1);
    bint32_lshift(aux, g, pow);
    bint32_and(aux, p, aux);
    bint32_xor(g, g, aux);
    bint32_lshift(aux0, p, pow);
    bint32_refresh(aux0, aux0);
    bint32_and(p, p, aux0);
  }
  bint32_lshift(aux, g, 1 << (LOG_BITSIZE - 1));
  bint32_and(aux, p, aux);
  bint32_xor(g, g, aux);
  bint32_lshift(aux, g, 1);
  bint32_xor(res, x, y);
  bint32_xor(res, res, aux);
  return;
}


// To share a public value
void bint32_to_base(bint32_t r, uint32_t a) {
  r[0] = a;
  memset(r+1, 0, (NUM_SHARES - 1) * sizeof(uint32_t));
  return;
}

// To share a secret value
void bint32_share_secret(bint32_t r, uint32_t a) {
  int i;
  r[0] = a;
  for (i = 1; i < NUM_SHARES; i++) {
    r[i] = uint32_rand();
    r[0] ^= r[i];
  }
  return;
}

void bint32_share_secret_modp(bint32_t r, uint32_t a) {
  int i;
  r[0] = a;
  for (i = 1; i < NUM_SHARES; i++) {
    r[i] = uint32_rand_modp();
    r[0] = (r[0] + Q - r[i]) % Q;
  }
  return;
}

// UNSAFE OPERATION should be protected
uint32_t bint32_recombine_share(bint32_t a) {
  uint32_t r;
  int i;

  r = a[0];
  for (i = 1; i < NUM_SHARES; i++) r ^= a[i];
  return r;
}

uint32_t bint32_full_xor(bint32_t a) {
  bint32_t b;
  bint32_full_refresh(b, a);
  return bint32_recombine_share(b);
}

void bint32_add_modp(bint32_t res, bint32_t x, bint32_t y) {
    static bint32_t p = { -Q, 0 };
    bint32_t s, s0, b;

    bint32_add(s, x, y);
    bint32_add(s0, s, p);
    bint32_rshift(b, s0, 31);
    bint32_neg(b, b);
    bint32_refresh(b, b);
    bint32_and(res, s, b);
    bint32_refresh(b, b);
    bint32_not(b, b);
    bint32_and(s, s0, b);
    bint32_xor(res, res, s);
}

void bint32_arith_bool_modp(bint32_t res, bint32_t a) {
    bint32_t b;
    int i;

    bint32_share_secret(res, a[0]);
    for (i = 1; i<NUM_SHARES; i++) {
	bint32_share_secret(b, a[i]);
	bint32_add_modp(res, res, b);
    }
}

uint32_t uint32_rand_modp() {
  const uint32_t maskP = (1 << Q_BITS) - 1;
  uint32_t r = -1;

  while(r >= Q) 
    r = uint32_rand() & maskP;

  return r;
}

void bint32_randomness_gen(bint32_t res) {
    static bint32_t k = { -2*B-1, 0 };
    bint32_t x, b, a, y;
    int i;

    const uint32_t maskK = (1 << (B_BITS+1)) - 1;
    
    do {
	for (i = 0; i<NUM_SHARES; i++) {
	    x[i] = uint32_rand() & maskK;
	}
	bint32_add(b, x, k);
	bint32_rshift(b, b, 31);
    }
    while(!bint32_recombine_share(b));

    for (i = 1; i<NUM_SHARES; i++) {
	res[i] = uint32_rand_modp();
	a[i] = Q - res[i];
    }
    a[0] = 0;

    bint32_arith_bool_modp(y, a);
    bint32_add_modp(y, x, y);
    res[0] = bint32_full_xor(y);
    if(res[0] >= B)
	res[0] -= B;
    else
	res[0] += Q-B;
}

uint32_t bint32_recombine_modp(bint32_t a) {
  uint32_t r = a[0];
  for(int i=1; i<NUM_SHARES; i++) {
    r += a[i];
    r %= Q;
  }
  return r;
}

uint32_t bint32_refresh_unmask(bint32_t a) {
  bint32_t b;
  bint32_refresh_modp(b, a);
  return bint32_recombine_modp(b);
}

int32_t bint32_recombine_modp_signed(bint32_t a) {
  int32_t r;
  r = bint32_recombine_modp(a);
  if(2*r >= Q)
    r -= Q;
  return r;
}

uint32_t bint32_all_large_enough(bint32_t *a, size_t count) {
  bint32_t r, a0, b, s;
  static bint32_t k0 = { B-OMEGA, 0 };
  static bint32_t p0 = { -(Q+1)/2, 0 };
  static bint32_t p  = { -Q, 0 };
  size_t j;

  bint32_to_base(r, 1);
  for(j = 0; j < count; j++) {
    bint32_arith_bool_modp(a0, a[j]);
    bint32_add(s, a0, p);
    bint32_add(b, a0, p0);
    bint32_rshift(b, b, 31);
    bint32_neg(b, b);
    bint32_refresh(b, b);
    bint32_and(a0, a0, b);
    bint32_not(b, b);
    bint32_refresh(b, b);
    bint32_and(s, s, b);
    bint32_xor(a0, a0, s);
    bint32_add(b, a0, k0);
    bint32_rshift(b, b, 31);
    bint32_not(b, b);
    bint32_and(r, r, b);
  }
  return bint32_full_xor(r);
}

void bint32_add_arith_modp(bint32_t r, bint32_t a, bint32_t b) {
  int i;
  for(i = 0; i < NUM_SHARES; i++) { 
    r[i] = (a[i] + b[i]) % Q; 
  }
}

void bint32_sub_arith_modp(bint32_t r, bint32_t a, bint32_t b) {
  int i;
  for(i = 0; i < NUM_SHARES; i++) { 
    r[i] = (a[i] + Q - b[i]) % Q; 
  }
}

void bint32_add_const_modp(bint32_t r, bint32_t a, uint32_t c) {
  bint32_copy(r, a);
  r[0] = (r[0]+c) % Q;
}

void bint32_sub_const_modp(bint32_t r, bint32_t a, uint32_t c) {
  bint32_neg_modp(r, a);
  r[0] = (r[0]+c) % Q;
}
