#ifndef HEADER_MASKING_H
#define HEADER_MASKING_H

#include <stdint.h>
#include <stdlib.h>
#include "glp_consts.h"

#define NUM_SHARES 3 
#define LOG_BITSIZE 5

typedef uint32_t bint32_t[NUM_SHARES];

#define bint32_copy(r,a) memcpy(r, a, NUM_SHARES * sizeof(uint32_t))

#define bint32_xor(r,a,b) \
  for(int i = 0;i < NUM_SHARES; i++) { \
    r[i]=a[i]^b[i]; \
  }

#define bint32_lshift(r, a, b)  \
  for (int i = 0; i < NUM_SHARES; i++) { \
    r[i] = a[i] << (b); \
  }

#define bint32_rshift(r, a, b)  \
  for (int i = 0; i < NUM_SHARES; i++) { \
    r[i] = a[i] >> (b); \
  }

#define bint32_neg(r, a)  \
  for (int i = 0; i < NUM_SHARES; i++) { \
    r[i] = -a[i]; \
  }

#define bint32_not(r, a)  \
  r[0] = ~a[0];  \
  for (int i = 1; i < NUM_SHARES; i++) { \
    r[i] = a[i]; \
  }

#define bint32_neg_modp(r, a)  \
  for (int i = 0; i < NUM_SHARES; i++) { \
    r[i] = Q-a[i]; \
  }

#define bint32_mul_const_modp(r, a, c)  \
  for (int i = 0; i < NUM_SHARES; i++) { \
    r[i] = (a[i]*c) % Q; \
  }

uint32_t uint32_rand();
uint32_t uint32_rand_modp();
uint32_t bint32_recombine_share(bint32_t a);
uint32_t bint32_full_xor(bint32_t a);
uint32_t bint32_all_large_enough(bint32_t *a, size_t count);
uint32_t bint32_refresh_unmask(bint32_t a);
int32_t bint32_recombine_modp_signed(bint32_t a);

void bint32_and(bint32_t rc, bint32_t a,bint32_t b);
void bint32_refresh(bint32_t r, bint32_t a);
void bint32_add (bint32_t res, bint32_t x, bint32_t y);
void bint32_to_base(bint32_t r, uint32_t a);
void bint32_share_secret(bint32_t r, uint32_t a);
void bint32_share_secret_modp(bint32_t r, uint32_t a);
void bint32_add_modp(bint32_t res, bint32_t x, bint32_t y);
void bint32_arith_bool_modp(bint32_t res, bint32_t a);
void bint32_randomness_gen(bint32_t res);
void bint32_add_arith_modp(bint32_t r, bint32_t a, bint32_t b);
void bint32_sub_arith_modp(bint32_t r, bint32_t a, bint32_t b);
void bint32_add_const_modp(bint32_t r, bint32_t a, uint32_t c);
void bint32_sub_const_modp(bint32_t r, bint32_t a, uint32_t c);

#endif /* HEADER_MASKING_H */
