/* This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * See LICENSE for complete information.
 */

#include <time.h>
#include "glp.h"
#include "glp_utils.h"
#include "test_vectors.h"
#define SIGN_TRIALS 100

int main(){

  double start, end;
  glp_signing_key_t sk;
  glp_masked_signing_key_t msk;
  glp_public_key_t pk;
  char *message = "testtest";
  glp_signature_t sig;

  uint16_t i;

  printf("**************************************************************\n");

  /*test a lot of verifications*/
  printf("trying %d independent keygen/sign/verifies\n", SIGN_TRIALS);
  start = (double)clock() /(double) CLOCKS_PER_SEC;
  for(i=0; i < SIGN_TRIALS; i++){
    glp_gen_sk(&sk);
    glp_gen_pk(&pk,sk);
    if(!glp_sign(&sig, sk,(unsigned char *)message,strlen(message))){
      printf("signature failure round %d!\n",i);
    }
    if(!glp_verify(sig, pk,(unsigned char *)message,strlen(message))){
      printf("verification failure round %d!\n",i);
      return 1;
    }
    if(!(i % 100)){
      printf("passed trial %d\n",i);
    }
  }
  end = (double)clock() /(double) CLOCKS_PER_SEC;
  printf("signature scheme validates across %d independent trials\n", SIGN_TRIALS);
  printf("time: %f s\n", end - start);
  printf("**************************************************************\n");

  printf("trying %d independent MASKED keygen/sign/verifies\n", SIGN_TRIALS);
  printf("(masking with %d shares)\n", NUM_SHARES);
  start = (double)clock() /(double) CLOCKS_PER_SEC;
  for(i=0; i < SIGN_TRIALS; i++){
    glp_gen_sk(&sk);
    glp_mask_sk(&msk, &sk);
    glp_gen_pk(&pk,sk);
    if(!glp_masked_sign(&sig, msk,(unsigned char *)message,strlen(message))){
      printf("signature failure round %d!\n",i);
    }
    if(!glp_verify(sig, pk,(unsigned char *)message,strlen(message))){
      printf("verification failure round %d!\n",i);
      return 1;
    }
    if(!(i % 100)){
      printf("passed trial %d\n",i);
    }
  }
  end = (double)clock() /(double) CLOCKS_PER_SEC;
  printf("MASKED signature scheme validates across %d independent trials\n", SIGN_TRIALS);
  printf("time: %f s\n", end - start);
  printf("**************************************************************\n");
  
  return 0;
}
